#//-----------------------------------------------
#//TUSB-0216ADM用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_05_24　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#デバイス用DLLのロード  
tusb16addll = cdll.LoadLibrary("TUSB16AD.DLL")


tusb16addll.TUSB0216AD_Device_Open.restype = c_short
tusb16addll.TUSB0216AD_Device_Open.argtypes = (c_short,)
tusb16addll.TUSB0216AD_Device_Close.restype = None
tusb16addll.TUSB0216AD_Device_Close.argtypes = (c_short,)
tusb16addll.TUSB0216AD_DIO_In.restype = c_short
tusb16addll.TUSB0216AD_DIO_In.argtypes = (c_short,POINTER(c_ubyte))
tusb16addll.TUSB0216AD_DIO_Out.restype = c_short
tusb16addll.TUSB0216AD_DIO_Out.argtypes = (c_short,c_ubyte)
tusb16addll.TUSB0216AD_DIO_Chk.restype = c_short
tusb16addll.TUSB0216AD_DIO_Chk.argtypes = (c_short,POINTER(c_ubyte))
tusb16addll.TUSB0216AD_Ad_Single.restype = c_short
tusb16addll.TUSB0216AD_Ad_Single.argtypes = (c_short,POINTER(c_int))
tusb16addll.TUSB0216AD_Start.restype = c_short
tusb16addll.TUSB0216AD_Start.argtypes = (c_short,c_ubyte,c_uint,c_ubyte,c_ubyte)
tusb16addll.TUSB0216AD_Stop.restype = c_short
tusb16addll.TUSB0216AD_Stop.argtypes = (c_short,)
tusb16addll.TUSB0216AD_Ad_Status.restype = c_short
tusb16addll.TUSB0216AD_Ad_Status.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte),POINTER(c_uint))
tusb16addll.TUSB0216AD_Ad_Data.restype = c_short
tusb16addll.TUSB0216AD_Ad_Data.argtypes = (c_short,c_ubyte,POINTER(c_uint),POINTER(c_uint))
tusb16addll.TUSB0216AD_AdClk_Set.restype = c_short
tusb16addll.TUSB0216AD_AdClk_Set.argtypes = (c_short,c_uint,c_ubyte)
tusb16addll.TUSB0216AD_Level_Set.restype = c_short
tusb16addll.TUSB0216AD_Level_Set.argtypes = (c_short,c_int,c_short)
tusb16addll.TUSB0216AD_Input_Set.restype = c_short
tusb16addll.TUSB0216AD_Input_Set.argtypes = (c_short,c_ubyte,c_ubyte)
tusb16addll.TUSB0216AD_Input_Check.restype = c_short
tusb16addll.TUSB0216AD_Input_Check.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte))
tusb16addll.TUSB0216AD_Trigger.restype = c_short
tusb16addll.TUSB0216AD_Trigger.argtypes = (c_short,)

#######  補助関数 (これも呼ばない)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

######　機能関数の定義 
######  以下の関数を呼び出してデバイスを操作する
#DevIdは装置の設定ID

def TUSB0216AD_Device_Open(DevId):
    """Open Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId) ):
        return tusb16addll.TUSB0216AD_Device_Open(DevId)
    else:
        return 8    #Invalid parameter
    
def TUSB0216AD_Device_Close(DevId):
    """Close Device Connection

    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    None
    """
    if( IsValidId(DevId) ):
        tusb16addll.TUSB0216AD_Device_Close(DevId)
    else:
        return 8    #Invalid parameter

def TUSB0216AD_DIO_In(DevId):
    """Read DIO Input port
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Input Data
    """
    if( IsValidId(DevId)):
        Data = c_ubyte()
        ret = tusb16addll.TUSB0216AD_DIO_In(DevId,byref(Data))
        if( ret != 0 ):
            return ret,0
        else:
            return 0,Data.value
    else:
        return 8,0  #Invalid parameter

def TUSB0216AD_DIO_Out(DevId,Data):
    """Write DIO Ouput port
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data            ----    Output Data 0~63
    
    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 8
    if(not IsInRange(Data,0,63)):return 8
    return tusb16addll.TUSB0216AD_DIO_Out(DevId,Data)

def TUSB0216AD_DIO_Chk(DevId):
    """Read DIO Output
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    int             ----    Output bit pattern
    """
    if(IsValidId(DevId)):
        Data = c_ubyte()
        ret = tusb16addll.TUSB0216AD_DIO_Chk(DevId,byref(Data))
        if(ret != 0):
            return ret,0
        else:
            return 0,Data.value
    else:
        return 8,0  #Invalid parameter

def TUSB0216AD_Ad_Single(DevId):
    """Get Ad Sampled Datas on demand
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Output bit pattern
    """
    if(IsValidId(DevId)):
        Data = c_ubyte()
        ret = tusb16addll.TUSB0216AD_Ad_Single(DevId,byref(Data))
        if(ret != 0):
            return ret,0
        else:
            return 0,Data.value
    else:
        return 8,0  #Invalid parameter

def TUSB0216AD_Start(DevId,Ch,PreLen,TrgType,TrgCh):
    """Start Continuous Sampling
    
    Arguments:
    DevId           ----    Device ID 0~15
    Ch              ----    Channels 0:1ch 1:2ch 2:1ch & 2ch
    PreLen          ----    Pre Trigger Length 0 ~ 262143
    TrgType         ----    Trigger type 0:Soft 1:Ext Digital 2:Analog Up Edge 3:Analog Down Edge
    TrgCh           ----    Analog Trigger Ch Select 0:ch1 1:ch2

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 8
    if(not IsInRange(Ch,0,2)):return 8
    if(not IsInRange(PreLen,0,262143)):return 8
    if(not IsInRange(TrgType,0,3)):return 8
    if(not IsInRange(TrgCh,0,1)):return 8
    return tusb16addll.TUSB0216AD_Start(DevId,Ch,PreLen,TrgType,TrgCh)

def TUSB0216AD_Stop(DevId):
    """Stop Continuous Sampling
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(IsValidId(DevId)):
        return tusb16addll.TUSB0216AD_Stop(DevId)
    else:
        return 8

def TUSB0216AD_Ad_Status(DevId):
    """Load Sampling Status
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    int             ----    Sampling Status 0:Stop 1:Waiting for trigger 2:Stop 3:Sampling
    int             ----    Overflow Status Ch1 0:none 1:Overflow detected
    int             ----    Overflow Status Ch2 0:none 1:Overflow detected
    int             ----    Sampled Data Length Ch1
    int             ----    Sampled Data Length Ch2
    """
    if(IsValidId(DevId)):
        Status = c_ubyte()
        OverFlow = (c_ubyte *2)()
        p_OverFlow = cast(pointer(OverFlow),POINTER(c_ubyte))
        DataLen = (c_uint *2)()
        p_DataLen = cast(pointer(DataLen),POINTER(c_uint))
        ret = tusb16addll.TUSB0216AD_Ad_Status(DevId,byref(Status),p_OverFlow,p_DataLen)
        if ret != 0:
            return ret,None,None,None,None,None
        else:
            return 0,Status.value,OverFlow[0],OverFlow[1],DataLen[0],DataLen[1]

def TUSB0216AD_Ad_Data(DevId,Ch,DataLen):
    """Get Sampled Data tuple
    
    Arguments:
    DevId           ----    Device ID 0~15
    Ch              ----    Channel 0 or 1
    DataLen         ----    Request Data Length 1~262144

    Returns:
    int             ----    Result code
    list[int]       ----    Data list
    """
    if(not IsValidId(DevId)):return 8,None
    if(not IsInRange(Ch,0,1)):return 8,None
    if(not IsInRange(DataLen,1,262144)): 8,None
    Data = (c_uint * DataLen)()
    p_Data = cast(pointer(Data),POINTER(c_uint))
    cDataLen = c_uint()
    cDataLen.value = DataLen
    ret = tusb16addll.TUSB0216AD_Ad_Data(DevId,Ch,p_Data,byref(cDataLen))
    if( ret != 0): return ret,None
    RData = []
    for x in Data[:cDataLen.value]:
        RData.append(x)
    return 0,RData

def TUSB0216AD_AdClk_Set(DevId,ClkTime,Sel):
    """Set AD Clock Settings
    
    Arguments:
    DevId           ----    Device ID 0~15
    ClkTime         ----    Internal Clock Period 500~2147483647
    Sel             ----    Clock Select 0:Internal 1:External

    Return:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 8
    if(not IsInRange(ClkTime,500,2147483647)):return 8
    if(not IsInRange(Sel,0,1)):return 8
    return tusb16addll.TUSB0216AD_AdClk_Set(DevId,ClkTime,Sel)

def TUSB0216AD_Level_Set(DevId,Level,Hys):
    """Set Analog Trigger Level
    
    Arguments:
    DevId           ----    Device ID 0~15
    Level           ----    Trigger Level 1~65534
    Hys             ----    Noise Rejection Hysteresis Level 0~660

    Return:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 8
    if(not IsInRange(Level,1,65534)):return 8
    if(not IsInRange(Hys,0,660)):return 8
    return tusb16addll.TUSB0216AD_Level_Set(DevId,Level,Hys)

def TUSB0216AD_Input_Set(DevId,Ch1Range,Ch2Range):
    """Set Analog Inputs
    
    Arguments:
    DevId           ----    Device ID 0~15
    Ch1Range        ----    Ch1 Input Range 0~6
    Ch2Range        ----    Ch2 Input Range 0~6

    Return:
    int             ----    Result code
    """
    if(not IsValidId(DevId)):return 8
    if(not IsInRange(Ch1Range,0,6)):return 8
    if(not IsInRange(Ch2Range,0,6)):return 8
    return tusb16addll.TUSB0216AD_Input_Set(DevId,Ch1Range,Ch2Range)

def TUSB0216AD_Input_Check(DevId):
    """Read Analog Input Settings
    
    Arguments:
    DevId           ----    Device ID 0~15

    Return:
    int             ----    Result code
    int             ----    Ch1 Input Range
    int             ----    Ch2 Input Range
    """
    if(IsValidId(DevId)):
        Ch1Range = c_ubyte()
        Ch2Range = c_ubyte()
        ret = tusb16addll.TUSB0216AD_Input_Check(DevId,byref(Ch1Range),byref(Ch2Range))
        if ret != 0:
            return ret,None,None
        else:
            return 0,Ch1Range.value,Ch2Range.value
    else:
        return 8,None,None
    
def TUSB0216AD_Trigger(DevId):
    """Execute Trigger
    
    Arguments:
    DevId           ----    Device ID 0~15

    Return:
    int             ----    Result code
    """
    if( IsValidId(DevId)):
        return tusb16addll.TUSB0216AD_Trigger(DevId)
    else:
        return 8
    
# Error Messages from return coddes
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバがインストールされていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            11:"連続取り込み動作中です",
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"

