#//-----------------------------------------------
#//TUSB-0216ADM用サンプルプログラム
#//Python 用モジュール
#//
#//2024_05_24　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSB16AD
import sys

from SpFuncs import *
from tkinter import filedialog

class Tusb16adm_Device:
    """TUSB-0216ADM デバイス操作クラス フォーム"""

    DevId = None            # デバイスID
    mon_t = None            # DIOモニタータイマー
    smpl_t = None           # AD連続サンプリングタイマー
    DFile = [None,None]     # サンプリングデータ保存ファイル
    ChEn = [False,False]    # 取り込みチャンネル有効フラグ
    SampledLen = [0,0]      # サンプリングデータ数
    DataLimit = 0           # 取り込みデータ数上限　0の時無制限
    SmplingErr = False      # サンプリング中にエラー発生

    def __init__(self,DevId):
        """イニシャライザ
        
        arguments:
        DevId           ----    Device id 0~15
        """
        self.formgen()
        ret = TUSB16AD.TUSB0216AD_Device_Open(DevId)
        if ret == 0:    #正常にオープン出来た
            self.DevId = DevId                                      # デバイスIDをクラスメンバ変数に保持する
            self.form.protocol("WM_DELETE_WINDOW", self.DevClose)   # フォームが閉じられる時の処理を指定する
            if self.DevInit():
                self.mon_t = intTT(0.5,self.Monitor)                # モニタータイマー作成、コールバック関数登録
                self.mon_t.Start()                                      # モニタータイマースレッド開始
                self.WidgetState("Stop")
                # ボタンおよびチェックボックス変更時のコマンドハンドラを設定する
                for bit in range(6):
                    self.BtnStart.config(command=self.Start_on)
                    self.BtnStop.config(command=self.Stop_on)
                    self.BtnTrg.config(command=self.Trigger_on)
                    self.ChkDout[bit].config(command=self.Check_on)
            else:
                TUSB16AD.TUSB0216AD_Device_Close(DevId)                  # デバイス初期化に失敗したらデバイスを閉じる
                self.form.destroy()                                     # フォームも閉じる
        else:
            ShowInfo(TUSB16AD.GetErrMessage(ret))                   # 失敗エラーメッセージ表示
            self.form.destroy()                                     # フォームを閉じる

    def DevInit(self):
        """デバイスの状態を確認して画面表示に反映する"""
        self.Din_Check()
        ret = TUSB16AD.TUSB0216AD_DIO_Chk(self.DevId)           # デジタル出力の設定を読み込む
        if ret[0] != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret[0]))
            return False
        for bit in range(6):
            self.ChkDoutV[bit].set(IsBitSet(ret[1],bit))        # デジタル出力の設定を画面に反映する
        ret = TUSB16AD.TUSB0216AD_Input_Check(self.DevId)       # アナログ入力のレンジ設定を読み込む
        if ret[0] != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret[0]))
            return False
        self.CmbbRng[0].current(ret[1])                         # レンジ設定を画面に反映する
        self.CmbbRng[1].current(ret[2])
        return True

    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.mon_t.Stop()                               # モニタタイマー終了
            TUSB16AD.TUSB0216AD_Device_Close(self.DevId)    # デバイスクローズ
            self.form.destroy()                             # フォームを閉じる
            self.DevId = None

    def WidgetState(self,stat):
        """状態に応じてウィジットの表示を切り替える処理
        
        Arguments
        stat            ----    設定状態を表す文字列

        'Stop' - デバイスが接続されていて連続取り込み実行していない時
        'Run' - デバイスが接続されていて連続取り込みを実行している時
        """
        EnStop = tk.NORMAL if( stat == 'Stop' ) else tk.DISABLED
        EnRun = tk.NORMAL if( stat == 'Run' ) else tk.DISABLED
        self.ChkCh[0]['state'] = EnStop
        self.ChkCh[1]['state'] = EnStop
        self.CmbbRng[0]['state'] = EnStop
        self.CmbbRng[1]['state'] = EnStop
        self.CmbbTrgType['state'] = EnStop
        self.TxtTrgLevel['state'] = EnStop
        self.CmbbTrgCh['state'] = EnStop
        self.RBtnClkSel[0]['state'] = EnStop
        self.RBtnClkSel[1]['state'] = EnStop
        self.TxtClkIntv['state'] = EnStop
        self.TxtPreLen['state'] = EnStop
        self.TxtSaveLen['state'] = EnStop
        self.BtnStart['state'] = EnStop
        self.BtnStop['state'] = EnRun
        self.BtnTrg['state'] = EnRun

    # 補助的関数---------------------------------------------
    def Din_Check(self):
        """Digial Inputの状態確認と表示"""
        ret = TUSB16AD.TUSB0216AD_DIO_In(self.DevId)        # デジタル入力の読み込み
        if ret[0] != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret[0]))
            return
        for bit in range(6):                                # 1ビット毎に画面に反映させる
            BitStat = IsBitSet(ret[1],bit)
            if BitStat != self.LedDinR[bit]:                # 変更された入力のみ表示書き換え実行
                self.LedDinR[bit] = BitStat
                SetLEDCanvas(self.LedDin[bit],BitStat)
    
    def SmplStop(self):
        """連続サンプリングの終了"""
        ret = TUSB16AD.TUSB0216AD_Stop(self.DevId)
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))
        self.smpl_t.Stop()                              # サンプリングタイマー停止
        self.smpl_t = None
        self.SaveFileClose()                            # ファイルを閉じる
        self.WidgetState("Stop")                        # フォームウィジェット表示をStop状態にする

    def AdCmpChecker(self):
        """取り込み完了を確認したらサンプリングを終了する"""
        if self.SmplingErr:
            self.SmplStop()
            return
        if self.smpl_t != None:
            if self.DataLimit != 0:
                if self.ChEn[0]:
                    if self.DataLimit > self.SampledLen[0]:
                        self.form.after(100,self.AdCmpChecker)  # 100ms待って繰り返す
                        return
                if self.ChEn[1]:
                    if self.DataLimit > self.SampledLen[1]:
                        self.form.after(100,self.AdCmpChecker)  # 100ms待って繰り返す
                        return
            self.SmplStop()     # 両方とも完了していればサンプル終了

    # イベントハンドラ---------------------------------------------
    def Start_on(self):
        """Start Button"""
        # チャンネルセレクト確認
        self.ChEn[0] = self.ChkChV[0].get()
        self.ChEn[1] = self.ChkChV[1].get()
        if self.ChEn[0] and self.ChEn[1]: # Ch1 & Ch2
            ChSel = 2
        elif self.ChEn[0]:                # Ch1
            ChSel = 0
        elif self.ChEn[1]:                # Ch2
            ChSel = 1
        else:
            ShowInfo("チャンネルが選択されていません")
            return
        # トリガタイプ トリガレベル確認
        TrgType = self.CmbbTrgType.current()
        if TrgType == 0 or TrgType == 1:
            TrgLevel = 32768
            TrgCh = 0
        else:
            TrgLevel = ForceToInt(self.TxtTrgLevel.get(),0)
            if TrgLevel < 1 or TrgLevel > 65534:
                ShowInfo("トリガレベル入力が不正です")
                return
            # トリガチャンネル
            TrgCh = self.CmbbTrgCh.current()

        # クロック選択確認
        ClkSel = self.RbtnClkSelV.get()
        if ClkSel == 0:
            # クロックインターバル
            ClkTime = ForceToInt(self.TxtClkIntv.get(),0)
            if (ClkTime < 500) or (ClkTime > 2147483647):
                ShowInfo("クロックインターバル入力が不正です")
                return
        else:
            ClkTime = 500
        ret = TUSB16AD.TUSB0216AD_AdClk_Set(self.DevId,ClkTime,ClkSel)
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))
            return
        # プレトリガ長確認
        PreLen = ForceToInt(self.TxtPreLen.get(),-1)
        if PreLen < 0 or PreLen > 262144:
            ShowInfo("プレトリガ設定値が不正です")
            return
        # 保存データ数確認
        self.DataLimit = ForceToInt(self.TxtSaveLen.get(),-1)
        if self.DataLimit < 0:
            ShowInfo("保存データ数が不正です")
            return
        ret = TUSB16AD.TUSB0216AD_Input_Set(self.DevId,self.CmbbRng[0].current(),self.CmbbRng[1].current())
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))
            return
        HysLevel = 100  # ヒステリシス
        # トリガレベル設定
        ret = TUSB16AD.TUSB0216AD_Level_Set(self.DevId,TrgLevel,HysLevel)
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))
            return
        if self.SaveFileOpen() != True: return
        # 連続サンプリング開始
        ret = TUSB16AD.TUSB0216AD_Start(self.DevId,ChSel,PreLen,TrgType,TrgCh)
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))
            self.SaveFileClose()
            return
        self.SmplingErr = False
        self.smpl_t = intTT(0.1,self.SampleT)       # サンプリングタイマー作成、コールバック関数指定
        self.smpl_t.Start()                         # サンプリングタイマースタート
        self.WidgetState("Run")                     # フォームウィジェット表示をRun状態にする
        self.AdCmpChecker()

    def Stop_on(self):
        """Stop Button"""
        self.SmplStop()

    def Trigger_on(self):
        """Trigger"""
        ret = TUSB16AD.TUSB0216AD_Trigger(self.DevId)
        if ret != 0:
            ShowInfo(TUSB16AD.GetErrMessage(ret))

    def Check_on(self):
        """出力ビットが変更された時のハンドラ"""
        OutBit = 0
        for bit in range(6):
            OutBit <<= 1
            OutBit += 1 if self.ChkDoutV[5-bit].get() else 0
        ret = TUSB16AD.TUSB0216AD_DIO_Out(self.DevId,OutBit)    # デジタル出力確認
        if ret != 0: ShowInfo(TUSB16AD.GetErrMessage(ret))

    # timer -------------------------------------
    def Monitor(self):
        """モニタータイマー DIO入力チェック"""
        self.Din_Check()

    def SampleT(self):
        """サンプリングタイマーコールバック"""
        self.DataReadAndSave()

    # ファイル入出力関連処理 ----------------------------------
    def SaveFileOpen(self):
        """保存ファイルの選択とオープン
        
        Returns:
        bool            ----    成功 True 失敗 False
        """
        filename = ["",""]
        for ch in range(2):
            if self.ChEn[ch]:
                filename[ch] = filedialog.asksaveasfilename(
                    title = "Ch" + str(ch+1) + "保存ファイルを指定してください",
                    filetypes = [("データファイル","*.csv")],
                    defaultextension = "csv"
                    )
                if len(filename[ch]) == 0:
                    if self.DFile[0] != None:
                        self.DFile[0].close()
                        self.DFile[0] = None
                    return False
                if filename[0] == filename[1]:
                    ShowInfo("ファイル名が重複しています")
                    if self.DFile[0] != None:
                        self.DFile[0].close()
                        self.DFile[0] = None
                    return False
                try:
                    self.DFile[ch] = open(filename[ch],"w")     # ファイルを開く
                    self.SampledLen[ch] = 0
                except OSError as e:
                    ShowInfo(e)
                    return False
        return True

    def SaveFileClose(self):
        """保存ファイルを閉じる"""
        for ch in range(2):
            if self.DFile[ch] != None:
                self.DFile[ch].close()                  # ファイルを閉じる
                self.DFile[ch] = None

    def SaveFileWrite(self,Data,Ch):
        """保存ファイルにデータを保存する
        
        Arguments:
        Data            ----    データタプル(or リスト)
        Ch              ----    チャンネル 0:ch1 1:ch2
        """
        if self.DFile[Ch] == None: return       # データファイルが無しならリターン
        datl = []
        for dat in Data:
            datl.append(str(dat))
        self.DFile[Ch].write("\n".join(datl)+"\n")
    
    def DataReadAndSave(self):
        """デバイスからデータを読み出してファイルに保存する"""
        ret = TUSB16AD.TUSB0216AD_Ad_Status(self.DevId)     # 連続取り込み状態確認
        if ret[0] != 0:
            # ShowInfo(TUSBK02AD.GetErrMessage(ret))
            return
        Status = ret[1]         # 取り込み状態
        Ovf = ret[2],ret[3]     # オーバーフロー状態
        Len = ret[4],ret[5]    # 取り込み保持データ数
        Leng = 0
        if Status == 1:         # トリガ待ち
            self.TxtboxStat['text'] = "トリガ待ちです"
            return
        elif Status == 3:   # サンプリング中
            #self.TxtboxStat['text'] = "サンプリング中です\n"
            for ch in range(2):
                if self.ChEn[ch]:
                    if Ovf[ch] != 0:
                        self.TxtboxStat['text'] += "ch" + str(ch) + "バッファオーバーフロー\n"
                        self.SmplingErr = True
                        return
                    Leng = Len[ch]
                    if ( self.DataLimit != 0 ) and (( self.SampledLen[ch] + Leng) >= self.DataLimit ): # 最大取り込み長を超えていたら
                        Leng = self.DataLimit - self.SampledLen[ch]
                    if Leng > 0:        # データが1個以上あれば
                        ret = TUSB16AD.TUSB0216AD_Ad_Data(self.DevId,ch,Leng)   # Leng個データを取得する
                        if ret[0] != 0:return
                        self.SaveFileWrite(ret[1],ch)                           # データをファイルに保存する
                        self.SampledLen[ch] += len(ret[1])
            self.TxtboxStat['text'] = ""
            for ch in range(2):
                if self.ChEn[ch]:
                    self.TxtboxStat['text'] += "ch" + str(ch) + "データ数 = " + str(self.SampledLen[ch]) + "\n"
        else:
            self.TxtboxStat['text'] = "停止中です"

    # フォームとウィジェットを生成する処理---------------------------
    def formgen(self):
        AdRangeDict= {
            "±10V":0,
            "±5V":1,
            "±2.5V":2,
            "±1.25V":3,
            "10V":4,
            "5V":5,
            "2.5V":6,
        }
        self.form = tk.Tk()
        self.form.geometry('550x486')
        self.form.title('TUSB-0216ADM Sample Program')

        # Ch Settings
        self.ChkCh = GenList(2,None)      # Ch選択チェックボックス
        self.ChkChV = GenTkVarList(2,tk.BooleanVar,False)  # Ch選択チェックボックスのチェック状態
        self.CmbbRng = GenList(2,None)    # レンジ選択コンボボックス
        GetLabel(self.form,"Ch.",0.05,0.05)
        GetLabel(self.form,"Range",0.15,0.05)
        self.ChkCh[0] = GetChkBox(self.form,"Ch1",0.05,0.1,self.ChkChV[0])
        self.ChkCh[1] = GetChkBox(self.form,"Ch2",0.05,0.15,self.ChkChV[1])
        self.CmbbRng[0] = GetComboBox(self.form,list(AdRangeDict.keys()),0.1,0.15,0.1)
        self.CmbbRng[1] = GetComboBox(self.form,list(AdRangeDict.keys()),0.1,0.15,0.15)

        # Sampling Settings
        TriggerTypeDict= {
            "Soft":0,
            "Ext. Digital":1,
            "Edge(Up)":2,
            "Edge(Down)":3,
        }
        TriggerChDict= {
            "1":0,
            "2":1,
        }
        ClkSelDict= {
            "Internal":0,
            "External":1,
        }
        ry = 0.05                   # 高さ位置調整用変数
        ypitch = 0.08               # このフレーム各行の高さピッチ
        GetLabel(self.form,"Trigger Select",0.3,ry)
        self.CmbbTrgType = GetComboBox(self.form,list(TriggerTypeDict.keys()),0.2,0.6,ry)
        self.BtnTrg = GetButton(self.form,"TRG",None,0.07,0.9,ry)

        ry += ypitch
        GetLabel(self.form,"Trigger Level(1-65534)",0.3,ry)
        self.TxtTrgLevel = GetEntry(self.form,"32768",0.1,0.6,ry)

        ry += ypitch
        GetLabel(self.form,"Trigger Ch.",0.3,ry)
        self.CmbbTrgCh = GetComboBox(self.form,list(TriggerChDict.keys()),0.1,0.6,ry)

        ry += ypitch
        self.RBtnClkSel = None                   # クロックソース選択ラジオボタン
        self.RbtnClkSelV = tk.IntVar(value=0)    # クロックソース選択ラジオボタンの値
        GetLabel(self.form,"Clock Select",0.3,ry)
        self.RBtnClkSel = GetRadioButtons(self.form,ClkSelDict,0.6,ry,0.15,0,self.RbtnClkSelV)

        ry += ypitch
        GetLabel(self.form,"Clock Interval\n(500-2147483647)",0.3,ry)['justify'] = tk.LEFT
        self.TxtClkIntv = GetEntry(self.form,"1000",0.2,0.6,ry)

        ry += ypitch
        GetLabel(self.form,"Pre-Trigger Leng.\n(0-262144)",0.3,ry)['justify'] = tk.LEFT
        self.TxtPreLen = GetEntry(self.form,"0",0.2,0.6,ry)

        ry += ypitch
        GetLabel(self.form,"Save Leng. Limit",0.3,ry)
        self.TxtSaveLen = GetEntry(self.form,"1000000",0.2,0.6,ry)

        ry += ypitch
        self.TxtboxStat = GetLabelDispML(self.form,"",0.5,0.1,0.3,ry)

        ry += 0.1
        self.BtnStart = GetButton(self.form,"Start",None,0.2,0.4,ry+0.05)
        self.BtnStop = GetButton(self.form,"Stop",None,0.2,0.65,ry+0.05)

        # DIO Frame
        self.ChkDout = GenList(6,None)      # Digital Out ビットチェックボックス
        self.ChkDoutV = GenTkVarList(6,tk.BooleanVar,False)  # Digital Out ビットチェックボックスのチェック状態
        self.LedDin = GenList(6,None)        # LEDイメージ表示器
        self.LedDinR = GenList(6,False)      # LEDイメージ表示状態保持変数
        frame_dio = GetLFrame(self.form,'Digital I/O',0.95,0.15,0.025,0.8)
        for bit in range(6):
            GetLabel(frame_dio,str(bit),0.11+0.05*bit,0.3)
            self.ChkDout[bit] = GetChkBox(frame_dio,"",0.1+0.05*bit,0.7,self.ChkDoutV[bit])
            GetLabel(frame_dio,str(bit),0.59+0.05*bit,0.3)
            self.LedDin[bit] = GetLEDCanvas(frame_dio,0.6+0.05*bit,0.7)
            SetLEDCanvas(self.LedDin[bit],False)

# 実行部 ---------------------------------
if __name__=="__main__":
    gDevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            gDevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusb16adm_Device(DevId=gDevId)
    tmp.form.mainloop()

